//
//  BFCameraProgressView.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/17.
//

import Foundation
import BFCommonKit

class BFCameraProgressView: BFProgressBaseView {
    
    lazy var collectionV : UICollectionView = {
        let flowLayout = UICollectionViewFlowLayout()
        flowLayout.minimumLineSpacing = 0
        flowLayout.minimumInteritemSpacing = 0
        flowLayout.scrollDirection = .horizontal
        flowLayout.itemSize = CGSize(width: self.frame.width, height: self.frame.height)
        
        let cc = UICollectionView(frame: CGRect(x: 0, y: 0, width: self.width, height: self.height), collectionViewLayout: flowLayout)
        cc.delegate = self
        cc.dataSource = self
        cc.bounces = false
        cc.register(UICollectionViewCell.self, forCellWithReuseIdentifier: String(describing: UICollectionViewCell.self))
        cc.showsVerticalScrollIndicator = false
        cc.showsHorizontalScrollIndicator = false
        cc.backgroundColor = UIColor.clear
        return cc
    }()
    
    override init(frame: CGRect) {
        super.init(frame: frame)

        addSubview(collectionV)
        
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// 添加缩略图
    /// - Parameter images:
    func addThumbImages() {
        DispatchQueue.main.async {
            self.collectionV.reloadData()
        }
    }
}

extension BFCameraProgressView: UICollectionViewDelegateFlowLayout, UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return recordItem?.thumbImgs.count ?? 0
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        var size : CGSize = .zero
        if let img = recordItem?.thumbImgs[indexPath.row] {
            if let sticker = recordItem?.videoStickers.first(where: { m in
                m.thumImgs?.contains(img) as? Bool ?? false
            }) {
                if img == sticker.thumImgs!.last {
                    size = CGSize(width: (sticker.timelineOut - sticker.timelineIn) * 70.0 / 5.0, height: 50)
                }else{
                    size = CGSize(width: 70.0, height: 50)
                }
            }
        }
        BFLog(1, message: "size \(size), \(indexPath.row)")
        return size
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: String(describing: UICollectionViewCell.self), for: indexPath)
        var imgview : UIImageView? = cell.contentView.viewWithTag(13000) as? UIImageView
        if  imgview == nil {
            imgview = UIImageView(frame: CGRect(x: 0, y: 0, width: 70, height: 50))
            imgview?.contentMode = .scaleAspectFit
            imgview?.tag = 13000
            cell.contentView.addSubview(imgview!)
        }
        
        imgview?.image = recordItem?.thumbImgs[indexPath.row]
        
        return cell
    }
}

extension BFCameraProgressView {
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        let totalW = recordItem?.mediaType == .Video ? (scrollView.contentSize.width - width) : (CGFloat(recordItem?.materialDuraion.seconds ?? 0) * thumbImageWidth / 2.0)
        if recordItem?.mediaType == .Video {
            if isDrag {
                dragScrollProgressHandle?(false, totalW > 0 ? min(Float(scrollView.contentOffset.x / totalW), 1) : 0)
            }
        } else if recordItem?.mediaType == .Image {
            if isDrag {
                if scrollView.contentOffset.x > ((CGFloat(recordItem?.materialDuraion.seconds ?? 0) * thumbImageWidth / 2.0) + 0.5) {
                    scrollView.contentOffset = CGPoint(x: (CGFloat(recordItem?.materialDuraion.seconds ?? 0) * thumbImageWidth / 2.0) + 0.5, y: 0)
                }
                dragScrollProgressHandle?(false, totalW > 0 ? min(Float(scrollView.contentOffset.x / totalW), 1) : 0)
            }
        }
    }

    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        isDrag = true
        let totalW = recordItem?.mediaType == .Video ? (scrollView.contentSize.width - width) : (CGFloat(recordItem?.materialDuraion.seconds ?? 0) * thumbImageWidth / 2.0)
        dragStartHandle?()
        dragScrollProgressHandle?(true, totalW > 0 ? min(Float(scrollView.contentOffset.x / totalW), 1) : 0)
    }

    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if !decelerate {
            let totalW = recordItem?.mediaType == .Video ? (scrollView.contentSize.width - width) : (CGFloat(recordItem?.materialDuraion.seconds ?? 0) * thumbImageWidth / 2.0)
            isDrag = false
            dragEndHandle?(totalW > 0 ? min(Float(scrollView.contentOffset.x / totalW), 1) : 0)
        }
    }

    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        let totalW = recordItem?.mediaType == .Video ? (scrollView.contentSize.width - width) : (CGFloat(recordItem?.materialDuraion.seconds ?? 0) * thumbImageWidth / 2.0)
        isDrag = false
        dragEndHandle?(totalW > 0 ? min(Float(scrollView.contentOffset.x / totalW), 1) : 0)
    }
}
